#ifndef ADSR_H
#define ADSR_H

#include "specimen.h"
#include "ticks.h"

/* envelope state flags */
typedef enum {
     ADSR_STATE_IDLE,
     ADSR_STATE_ATTACK,
     ADSR_STATE_DECAY,
     ADSR_STATE_SUSTAIN,
     ADSR_STATE_RELEASE
} ADSRState;

/* envelope parameters */
typedef struct adsr_params_s {
     float attack;		/* attack length in seconds */
     float decay;		/* decay length in seconds */
     float sustain;		/* sustain level [0.0, 1.0] */
     float release;		/* release length in seconds */
} adsr_params_t;

/* adsr envelope structure */
typedef struct adsr_s {
     ADSRState    state;	/* what phase of the envelope we are in */
     tick_t       ticks;	/* how far along we are in the current state */
     float        val;		/* our current value [0.0, 1.0] */
     float        rval;		/* value we released at */
     tick_t       attack;	/* attack length in ticks */
     tick_t       decay;	/* decay length in ticks */
     float        sustain;	/* sustain level [0.0, 1.0] */
     tick_t       release;	/* release length in ticks */
} adsr_t;

void  adsr_init       (adsr_t *env);
void  adsr_trigger    (adsr_t *env);
void  adsr_release    (adsr_t *env);
float adsr_tick       (adsr_t *env);
void  adsr_set_params (adsr_t *env, adsr_params_t *params);

#endif /* ADSR_H */
