#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libxml/parser.h>
#include <glib.h>
#include "specimen.h"
#include "patch.h"

int beef_write(char *name)
{
     char tmp[256];
     char *ptmp;
     float tmpf;
     int val;
     int *id;
     int i, count;
     xmlDocPtr doc;
     xmlNodePtr rootnode;
     xmlNodePtr xmlpatch;

     debug("Writing current bank to file %s\n", name);

     /* create the guts of the doc */
     doc = xmlNewDoc("1.0");
     rootnode = xmlNewDocNode(doc, NULL, (const xmlChar *)"beef", NULL);
     xmlDocSetRootElement(doc, rootnode);

     /* walk the walk */
     count = patch_dump(&id);
     for (i = 0; i < count; i++) {
	  xmlpatch = xmlNewTextChild(rootnode, NULL, (const xmlChar *)"patch", NULL);

	  ptmp = patch_get_name(id[i]);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"name", (const xmlChar *)ptmp);
	  g_free(ptmp);

	  ptmp = patch_get_sample_name(id[i]);
	  if (ptmp[0] != '\0')
	       xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"file",
			       (const xmlChar *)ptmp);
	  g_free(ptmp);
	  
	  sprintf(tmp, "%d", patch_get_channel(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"channel", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_note(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"note", (const xmlChar *)tmp);
	  
	  sprintf(tmp, "%f", patch_get_volume(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"volume", (const xmlChar *)tmp);
	  
	  sprintf(tmp, "%f", patch_get_panning(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"pan", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_play_mode(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"play_mode", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_cut(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cut", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_cut_by(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cut_by", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_range(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"range", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_lower_note(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"lower_note", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_upper_note(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"upper_note", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_sample_start(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"sample_start", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_sample_stop(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"sample_stop", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_loop_start(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"loop_start", (const xmlChar *)tmp);

	  sprintf(tmp, "%d", patch_get_loop_stop(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"loop_stop", (const xmlChar *)tmp);

	  sprintf(tmp, "%f", patch_get_cutoff(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cutoff", (const xmlChar *)tmp);

	  sprintf(tmp, "%f", patch_get_resonance(id[i]));
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"resonance", (const xmlChar *)tmp);

	  /* volume */
	  patch_get_env_attack(id[i], PATCH_PARAM_VOLUME, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"volume_a", (const xmlChar *)tmp);

	  patch_get_env_decay(id[i], PATCH_PARAM_VOLUME, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"volume_d", (const xmlChar *)tmp);

	  patch_get_env_sustain(id[i], PATCH_PARAM_VOLUME, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"volume_s", (const xmlChar *)tmp);

	  patch_get_env_release(id[i], PATCH_PARAM_VOLUME, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"volume_r", (const xmlChar *)tmp);

	  /* panning */
	  patch_get_env_attack(id[i], PATCH_PARAM_PANNING, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"panning_a", (const xmlChar *)tmp);

	  patch_get_env_decay(id[i], PATCH_PARAM_PANNING, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"panning_d", (const xmlChar *)tmp);

	  patch_get_env_sustain(id[i], PATCH_PARAM_PANNING, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"panning_s", (const xmlChar *)tmp);

	  patch_get_env_release(id[i], PATCH_PARAM_PANNING, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"panning_r", (const xmlChar *)tmp);

	  patch_get_env_amount(id[i], PATCH_PARAM_PANNING, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"panning_amt", (const xmlChar *)tmp);

	  /* cutoff */
	  patch_get_env_attack(id[i], PATCH_PARAM_CUTOFF, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cutoff_a", (const xmlChar *)tmp);

	  patch_get_env_decay(id[i], PATCH_PARAM_CUTOFF, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cutoff_d", (const xmlChar *)tmp);

	  patch_get_env_sustain(id[i], PATCH_PARAM_CUTOFF, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cutoff_s", (const xmlChar *)tmp);

	  patch_get_env_release(id[i], PATCH_PARAM_CUTOFF, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cutoff_r", (const xmlChar *)tmp);

	  patch_get_env_amount(id[i], PATCH_PARAM_CUTOFF, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"cutoff_amt", (const xmlChar *)tmp);

	  /* resonance */
	  patch_get_env_attack(id[i], PATCH_PARAM_RESONANCE, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"resonance_a", (const xmlChar *)tmp);

	  patch_get_env_decay(id[i], PATCH_PARAM_RESONANCE, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"resonance_d", (const xmlChar *)tmp);

	  patch_get_env_sustain(id[i], PATCH_PARAM_RESONANCE, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"resonance_s", (const xmlChar *)tmp);

	  patch_get_env_release(id[i], PATCH_PARAM_RESONANCE, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"resonance_r", (const xmlChar *)tmp);

	  patch_get_env_amount(id[i], PATCH_PARAM_RESONANCE, &tmpf);
	  sprintf(tmp, "%f", tmpf);
	  xmlNewTextChild(xmlpatch, NULL, (const xmlChar *)"resonance_amt", (const xmlChar *)tmp);
	  
     }
     
     /* write the file to disk */
     val = xmlSaveFormatFile(name, doc, 1);
     xmlFreeDoc(doc);

     return val;
}

int beef_read(char *path)
{
     xmlDocPtr doc;
     xmlNodePtr rootnode;
     xmlNodePtr xmlpatch;
     xmlNodePtr cur;
     xmlChar *key;
     int id;

     debug("Loading bank from file %s\n", path);

     doc = xmlParseFile(path);
     if (doc == NULL) {
	  errmsg("Failed to parse %s\n", path);
	  return -1;
     }

     rootnode = xmlDocGetRootElement(doc);
     if (rootnode == NULL) {
	  errmsg("%s is empty\n", path);
	  xmlFreeDoc(doc);
	  return -1;
     }

     if (xmlStrcmp(rootnode->name, (const xmlChar *)"beef") != 0) {
	  errmsg("%s is not a valid 'beef' file\n", path);
	  xmlFreeDoc(doc);
	  return -1;
     }


     patch_destroy_all();
     for (xmlpatch = rootnode->xmlChildrenNode; xmlpatch != NULL;
	  xmlpatch = xmlpatch->next) {

	  key = NULL;
	  if (!xmlStrcmp(xmlpatch->name, (const xmlChar *)"patch")) {
	       id = patch_create("Loading...");

	       for (cur = xmlpatch->xmlChildrenNode; cur != NULL; cur = cur->next) {
		    if (!xmlStrcmp(cur->name, (const xmlChar *)"name")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_name(id, (char *)key);
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"file")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_sample_load(id, (char *)key);
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"channel")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_channel(id, atoi((const char *)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"note")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_note(id, atoi((const char *)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"volume")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_volume(id, atof((const char *)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"pan")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_panning(id, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"play_mode")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_play_mode(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cut")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_cut(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cut_by")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_cut_by(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"range")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_range(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"lower_note")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_lower_note(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"upper_note")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_upper_note(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"sample_start")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_sample_start(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"sample_stop")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_sample_stop(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"loop_start")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_loop_start(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"loop_stop")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_loop_stop(id, atoi((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cutoff")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_cutoff(id, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"resonance")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_resonance(id, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"volume_a")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_attack(id, PATCH_PARAM_VOLUME, atof((const char*)key));

		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"volume_d")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_decay(id, PATCH_PARAM_VOLUME, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"volume_s")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_sustain(id, PATCH_PARAM_VOLUME, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"volume_r")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_release(id, PATCH_PARAM_VOLUME, atof((const char*)key));

		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cutoff_a")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_attack(id, PATCH_PARAM_CUTOFF, atof((const char*)key));

		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cutoff_d")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_decay(id, PATCH_PARAM_CUTOFF, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cutoff_s")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_sustain(id, PATCH_PARAM_CUTOFF, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cutoff_r")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_release(id, PATCH_PARAM_CUTOFF, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"cutoff_amt")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_amount(id, PATCH_PARAM_CUTOFF, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"resonance_a")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_attack(id, PATCH_PARAM_RESONANCE, atof((const char*)key));

		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"resonance_d")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_decay(id, PATCH_PARAM_RESONANCE, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"resonance_s")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_sustain(id, PATCH_PARAM_RESONANCE, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"resonance_r")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_release(id, PATCH_PARAM_RESONANCE, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"resonance_amt")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_amount(id, PATCH_PARAM_RESONANCE, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"panning_a")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_attack(id, PATCH_PARAM_PANNING, atof((const char*)key));

		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"panning_d")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_decay(id, PATCH_PARAM_PANNING, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"panning_s")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_sustain(id, PATCH_PARAM_PANNING, atof((const char*)key));
		    
		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"panning_r")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_release(id, PATCH_PARAM_PANNING, atof((const char*)key));

		    } else if (!xmlStrcmp(cur->name, (const xmlChar *)"panning_amt")) {
			 key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			 patch_set_env_amount(id, PATCH_PARAM_PANNING, atof((const char*)key));
		    }
		    
		    if (key != NULL) {
			 xmlFree(key);
			 key = NULL;
		    }
	       }
	  }
     }

     xmlFreeDoc(doc);
     return 0;
}

