/*
 * piano.h - Piano canvas item
 *
 * Based on SwamiUiPiano.h
 * Copyright (C) 1999-2002 Josh Green
 *
 * Modifed for Specimen by Loki Davison <ltdav1@student.monash.edu.au>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __PIANO_H__
#define __PIANO_H__

#include <gtk/gtk.h>
#include <libgnomecanvas/libgnomecanvas.h>


typedef struct _Piano Piano;
typedef struct _PianoClass PianoClass;

#define TYPE_PIANO   (piano_get_type ())
#define PIANO(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PIANO, Piano))
#define PIANO_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PIANO, \
   PianoClass))
#define IS_PIANO(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PIANO))
#define IS_PIANO_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PIANO))

#define PIANO_DEFAULT_WIDTH  676 /* default width in pixels */
#define PIANO_DEFAULT_HEIGHT 48	/* default hight in pixels */
#define PIANO_DEFAULT_LOWER_OCTAVE 3 /* default lower keys octave */
#define PIANO_DEFAULT_UPPER_OCTAVE 4 /* default upper keys octave */

/*  Piano Object */
struct _Piano
{
  GnomeCanvasGroup parent_instance; /* derived from GnomeCanvasGroup */

  int width, height;		/* width and height in pixels */
  gpointer key_info; /* array of key info structures (see Piano.c) */
  GnomeCanvasItem *white_bg;	/* white background canvas item */
  GnomeCanvasItem *grey_edge;	/* white key grey shadow edge item */
  GnomeCanvasItem *outline;	/* piano outline */
  guint8 key_count;		/* number of keys */
  guint8 start_note;		/* note piano starts on (always note C) */
  guint8 lower_octave;		/* lower computer keyboard octave # */
  guint8 upper_octave;		/* upper computer keyboard octave # */
  guint8 mouse_note;		/* mouse selected note >127 = none */
  guint8 reserved;

  /* cached values */
  int white_count;
  gboolean up2date;		/* are variables below up to date? */
  double world_width, world_height;
  double whitekey_width;
  double grey_top;
  double black_half, black_height;
  double vline_width, hline_width;
};

struct _PianoClass
{
  GnomeCanvasGroupClass parent_class;

  void (*note_on) (Piano *piano, guint keynum);
  void (*note_off) (Piano *piano, guint keynum);
};

GType piano_get_type (void);
GtkWidget* piano_new (void);	

void piano_draw_range (Piano *piano);

void piano_set_octave (Piano *piano,
			       int oct_lower, int oct_upper);
int piano_key_to_note (Piano *piano, int key);
void piano_note_on (Piano *piano, int note, int button);
void piano_note_off (Piano *piano, int note);
int piano_pos_to_note (Piano *piano, double x, double y,
			       gboolean *isblack);
double piano_note_to_pos (Piano *piano, int note,
				  gboolean *isblack);
void piano_dnd(GtkWidget *widget, GdkDragContext *context,
                     gint x, gint y, GtkSelectionData *selection_data, guint info, guint time,
                     gpointer data);
GSList *uri_list_to_gslist(char *uri_list);
	
#endif
